import cffi

ffi = cffi.FFI()
ffi.set_source(
    "TB2J_OpenMX.cmod._scfout_parser", r"""

/**********************************************************************
  read_scfout.c:

  read_scfout.c is a subroutine to read a binary file,
  filename.scfout.
 
  Log of read_scfout.c:

  02/Jul./2003  Released by T. Ozaki
  22/Aug./2011  Released by H. Kotaka
  17/Sep./2019  Modified by N. Yamaguchi

 ***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#define MAX_LINE_SIZE 256
#define fp_bsize         1048576     /* buffer size for setvbuf */
#define SCFOUT_VERSION 3

void free_scfout();

int T_NumOrbs; /* total number of orbs*/
int *MP ;         /* Full system version of MP */
double ****HR;
double ****HR_imag;
double ***SR;




/*******************************************************
  int atomnun;
  the number of total atoms
 *******************************************************/
int atomnum;   

/*******************************************************
  int Catomnun;
  the number of atoms in the central region
 *******************************************************/
int Catomnum;

/*******************************************************
  int Latomnun;
  the number of atoms in the left lead
 *******************************************************/
int Latomnum;

/*******************************************************
  int Ratomnun;
  the number of atoms in the left lead
 *******************************************************/
int Ratomnum;   

/*******************************************************
  int SpinP_switch;
0: non-spin polarized 
1: spin polarized
3: non-collinear calculation
 *******************************************************/
int SpinP_switch;

/*******************************************************
  int TCpyCell;
  the total number of periodic cells
 *******************************************************/
int TCpyCell;

/*******************************************************
  int Solver;
  method for solving eigenvalue problem
 *******************************************************/
int Solver;

/*******************************************************
  double ChemP;
  chemical potential
 *******************************************************/
double ChemP;

/*******************************************************
  int Valence_Electrons;
  total number of valence electrons
 *******************************************************/
int Valence_Electrons;

/*******************************************************
  double Total_SpinS;
  total value of Spin (2*Total_SpinS = muB)
 *******************************************************/
double Total_SpinS;

/*******************************************************
  double E_Temp;
  electronic temperature
 *******************************************************/
double E_Temp;

/*******************************************************
  int *Total_NumOrbs; 
  the number of atomic orbitals in each atom
size: Total_NumOrbs[atomnum+1]
 *******************************************************/
int *Total_NumOrbs;

/*******************************************************
  int *FNAN; 
  the number of first neighboring atoms of each atom
size: FNAN[atomnum+1]
 *******************************************************/
int *FNAN;

/*******************************************************
  int **natn; 
  grobal index of neighboring atoms of an atom ct_AN
size: natn[atomnum+1][FNAN[ct_AN]+1]
 *******************************************************/
int **natn;

/*******************************************************
  int **ncn; 
  grobal index for cell of neighboring atoms of
  an atom ct_AN
size: ncn[atomnum+1][FNAN[ct_AN]+1]
 *******************************************************/
int **ncn;

/*******************************************************
  double **atv;
  x,y,and z-components of translation vector of  
  periodically copied cells
size: atv[TCpyCell+1][4];
 *******************************************************/
double **atv;

/*******************************************************
  int **atv_ijk;
  i,j,and k number of periodically copied cells
size: atv_ijk[TCpyCell+1][4];
 *******************************************************/
int **atv_ijk;

/*******************************************************
  double tv[4][4];
  unit cell vectors in Bohr
 *******************************************************/
double tv[4][4];

/*******************************************************
  double rtv[4][4]:
  reciprocal unit cell vectors in Bohr^{-1}

note:
tv_i \dot rtv_j = 2PI * Kronecker's delta_{ij}
 *******************************************************/
double rtv[4][4];

/*******************************************************
  double Gxyz[atomnum+1][60];
  atomic coordinates in Bohr
 *******************************************************/
double **Gxyz;

/*******************************************************
  double *****Hks;
  Kohn-Sham matrix elements of basis orbitals
size: Hks[SpinP_switch+1]
[atomnum+1]
[FNAN[ct_AN]+1]
[Total_NumOrbs[ct_AN]]
[Total_NumOrbs[h_AN]] 
 *******************************************************/
double *****Hks;

/*******************************************************
  double *****iHks;
  imaginary Kohn-Sham matrix elements of basis orbitals
  for alpha-alpha, beta-beta, and alpha-beta spin matrices
  of which contributions come from spin-orbit coupling 
  and Hubbard U effective potential.
size: iHks[3]
[atomnum+1]
[FNAN[ct_AN]+1]
[Total_NumOrbs[ct_AN]]
[Total_NumOrbs[h_AN]] 
 *******************************************************/
double *****iHks;

/*******************************************************
  double ****OLP;
  overlap matrix
size: OLP[atomnum+1]
[FNAN[ct_AN]+1]
[Total_NumOrbs[ct_AN]]
[Total_NumOrbs[h_AN]]
 *******************************************************/
double ****OLP;

/*******************************************************
  double *****DM;
  density matrix
size: DM[SpinP_switch+1]
[atomnum+1]
[FNAN[ct_AN]+1]
[Total_NumOrbs[ct_AN]]
[Total_NumOrbs[h_AN]]
 *******************************************************/
double *****DM;

/*******************************************************
  double *****iDM;
  density matrix
size: iDM[2]
[atomnum+1]
[FNAN[ct_AN]+1]
[Total_NumOrbs[ct_AN]]
[Total_NumOrbs[h_AN]]
 *******************************************************/
double *****iDM;

/*******************************************************
  double dipole_moment_core[4];
 *******************************************************/
double dipole_moment_core[4];

/*******************************************************
  int version; (added by N. Yamaguchi)
 *******************************************************/
int version;

/*******************************************************
  int order_max; (added by N. Yamaguchi for HWC)
 *******************************************************/
int order_max;

/*******************************************************
  double *cc_vec; (added by N. Yamaguchi for HWC)
 *******************************************************/
double *cc_vec;

/*******************************************************
  double ******OLPpo; (added by N. Yamaguchi for HWC)
 *******************************************************/
double ******OLPpo;

/*******************************************************
  double *****OLPmo; 
 *******************************************************/
double *****OLPmo;

/*******************************************************
  double dipole_moment_background[4];
 *******************************************************/
double dipole_moment_background[4];




/* Added by N. Yamaguchi ***/
/* ***
 Note: Use "FREAD" instead of "fread" to avoid the mismatch of endianness. 
 Every "fread" in "read_scfout.c" was replaced with "FREAD".
 *** */
#define LATEST_VERSION 3
#define FREAD(POINTER, SIZE, NUMBER, FILE_POINTER)\
  do {\
    fread(POINTER, SIZE, NUMBER, FILE_POINTER);\
    if (conversionSwitch){\
      int dATA;\
      for (dATA=0; dATA<NUMBER; dATA++){\
	char *out=(char*)(POINTER+dATA);\
	int bYTE;\
	for (bYTE=0; bYTE<SIZE/2; bYTE++){\
	  char tmp=out[bYTE];\
	  out[bYTE]=out[SIZE-bYTE-1];\
	  out[SIZE-bYTE-1]=tmp;\
	}\
      }\
    }\
  } while (0)
/* ***/

static void Input( FILE *fp );



void read_scfout(char *argv[])
{
  static FILE *fp;

  char buf[fp_bsize];          /* setvbuf */


  /* Added by N. Yamaguchi ***/
  /* Modified by N. Yamaguchi ***/

  if ((fp = fopen(argv[1],"r")) != NULL){

    setvbuf(fp,buf,_IOFBF,fp_bsize);  /* setvbuf */

      printf("\nRead the scfout file (%s)\n",argv[1]);fflush(stdout);

    Input(fp);
    fclose(fp);
  }

  else {
    printf("Failure of reading the scfout file (%s).\n",argv[1]);fflush(stdout);
  }

}


void Input( FILE *fp )
{
  static int Gc_AN,ct_AN,h_AN,i,j,can,Gh_AN;
  static int wan1,wan2,TNO1,TNO2,spin,Rn,num_lines;
  static int k,q_AN,Gq_AN;
  static int i_vec[20],*p_vec;
  static double d_vec[20];
  static char makeinp[100];
  static char strg[MAX_LINE_SIZE];
  int direction, order;
  FILE *fp_makeinp;
  char buf[fp_bsize];          /* setvbuf */

  /****************************************************
    atomnum
    spinP_switch
    version (added by N. Yamaguchi)
   ****************************************************/

  fread(i_vec,sizeof(int),6,fp);

  /* Disabled by N. Yamaguchi
   * atomnum      = i_vec[0];
   * SpinP_switch = i_vec[1];
   * Catomnum =     i_vec[2];
   * Latomnum =     i_vec[3];
   * Ratomnum =     i_vec[4];
   * TCpyCell =     i_vec[5];
   */

  /* Added by N. Yamaguchi ***/
  int conversionSwitch;
  if (i_vec[1]==0 && i_vec[1]<0 || i_vec[1]>(LATEST_VERSION)*4+3){
    conversionSwitch=1;
    int i;
    for (i=0; i<6; i++){
      int value=*(i_vec+i);
      char *in=(char*)&value;
      char *out=(char*)(i_vec+i);
      int j;
      for (j=0; j<sizeof(int); j++){
	out[j]=in[sizeof(int)-j-1];
      }
    }
    if (i_vec[1]==0 && i_vec[1]<0 || i_vec[1]>(LATEST_VERSION)*4+3){
      puts("Error: Mismatch of the endianness");fflush(stdout);
    }
  } else {
    conversionSwitch=0;
  }
  /* ***/

  atomnum      = i_vec[0];

  /* Disabled by N. Yamaguchi ***
  SpinP_switch = i_vec[1];
  * ***/

  /* Added by N. Yamaguchi ***/
  SpinP_switch=i_vec[1]%4;
  version=i_vec[1]/4;


  char* openmxVersion;

  if (version==0){
    openmxVersion="3.7, 3.8 or an older distribution";
  } else if (version==1){
    openmxVersion="3.7.x (for development of HWC)";
  } else if (version==2){
    openmxVersion="3.7.x (for development of HWF)";
  } else if (version==3){
    openmxVersion="3.9";
  }

  if (version!=SCFOUT_VERSION){
      printf("The file format of the SCFOUT file:  %d\n", version);
      printf("The vesion is not supported by the current read_scfout\n");
    exit(0);
  }

      //printf("SpinP_switch %d", SpinP_switch);

    puts("***");

    printf("The file format of the SCFOUT file:  %d\n", version);
    //puts("And it supports the following functions:");
    //puts("- jx");
    //puts("- polB");
    //puts("- kSpin");
    //puts("- Z2FH");
    //puts("- calB");
    puts("***");

  /* ***/

  Catomnum =     i_vec[2];
  Latomnum =     i_vec[3];
  Ratomnum =     i_vec[4];
  TCpyCell =     i_vec[5];

  /****************************************************
    order_max (added by N. Yamaguchi for HWC)
   ****************************************************/

  FREAD(i_vec, sizeof(int), 1, fp);
  order_max=i_vec[0];

  /****************************************************
    allocation of arrays:

    double atv[TCpyCell+1][4];
  ****************************************************/

  atv = (double**)malloc(sizeof(double*)*(TCpyCell+1));
  for (Rn=0; Rn<=TCpyCell; Rn++){
    atv[Rn] = (double*)malloc(sizeof(double)*4);
  }

  /****************************************************
    read atv[TCpyCell+1][4];
   ****************************************************/

  for (Rn=0; Rn<=TCpyCell; Rn++){
    FREAD(atv[Rn],sizeof(double),4,fp);
  }

  /****************************************************
    allocation of arrays:

    int atv_ijk[TCpyCell+1][4];
   ****************************************************/

  atv_ijk = (int**)malloc(sizeof(int*)*(TCpyCell+1));
  for (Rn=0; Rn<=TCpyCell; Rn++){
    atv_ijk[Rn] = (int*)malloc(sizeof(int)*4);
  }

  /****************************************************
    read atv_ijk[TCpyCell+1][4];
   ****************************************************/

  for (Rn=0; Rn<=TCpyCell; Rn++){
    FREAD(atv_ijk[Rn],sizeof(int),4,fp);
  }

  /****************************************************
    allocation of arrays:

    int Total_NumOrbs[atomnum+1];
    int FNAN[atomnum+1];
   ****************************************************/

  Total_NumOrbs = (int*)malloc(sizeof(int)*(atomnum+1));
  FNAN = (int*)malloc(sizeof(int)*(atomnum+1));

  /****************************************************
    the number of orbitals in each atom
   ****************************************************/

  p_vec = (int*)malloc(sizeof(int)*atomnum);
  FREAD(p_vec,sizeof(int),atomnum,fp);
  Total_NumOrbs[0] = 1;
  for (ct_AN=1; ct_AN<=atomnum; ct_AN++){
    Total_NumOrbs[ct_AN] = p_vec[ct_AN-1];
  }
  free(p_vec);

  /****************************************************
    FNAN[]:
    the number of first nearest neighbouring atoms
   ****************************************************/

  p_vec = (int*)malloc(sizeof(int)*atomnum);
  FREAD(p_vec,sizeof(int),atomnum,fp);
  FNAN[0] = 0;
  for (ct_AN=1; ct_AN<=atomnum; ct_AN++){
    FNAN[ct_AN] = p_vec[ct_AN-1];
  }
  free(p_vec);

  /****************************************************
    allocation of arrays:

    int natn[atomnum+1][FNAN[ct_AN]+1];
    int ncn[atomnum+1][FNAN[ct_AN]+1];
   ****************************************************/

  natn = (int**)malloc(sizeof(int*)*(atomnum+1));
  for (ct_AN=0; ct_AN<=atomnum; ct_AN++){
    natn[ct_AN] = (int*)malloc(sizeof(int)*(FNAN[ct_AN]+1));
  }

  ncn = (int**)malloc(sizeof(int*)*(atomnum+1));
  for (ct_AN=0; ct_AN<=atomnum; ct_AN++){
    ncn[ct_AN] = (int*)malloc(sizeof(int)*(FNAN[ct_AN]+1));
  }

  /****************************************************
    natn[][]:
    grobal index of neighboring atoms of an atom ct_AN
   ****************************************************/

  for (ct_AN=1; ct_AN<=atomnum; ct_AN++){
    FREAD(natn[ct_AN],sizeof(int),FNAN[ct_AN]+1,fp);
  }

  /****************************************************
    ncn[][]:
    grobal index for cell of neighboring atoms
    of an atom ct_AN
   ****************************************************/

  for (ct_AN=1; ct_AN<=atomnum; ct_AN++){
    FREAD(ncn[ct_AN],sizeof(int),FNAN[ct_AN]+1,fp);
  }

  /****************************************************
    tv[4][4]:
    unit cell vectors in Bohr
   ****************************************************/

  FREAD(tv[1],sizeof(double),4,fp);
  FREAD(tv[2],sizeof(double),4,fp);
  FREAD(tv[3],sizeof(double),4,fp);

  /****************************************************
    rtv[4][4]:
    unit cell vectors in Bohr
   ****************************************************/

  FREAD(rtv[1],sizeof(double),4,fp);
  FREAD(rtv[2],sizeof(double),4,fp);
  FREAD(rtv[3],sizeof(double),4,fp);

  /****************************************************
    Gxyz[][1-3]:
    atomic coordinates in Bohr
   ****************************************************/

  Gxyz = (double**)malloc(sizeof(double*)*(atomnum+1));
  for (i=0; i<(atomnum+1); i++){
    Gxyz[i] = (double*)malloc(sizeof(double)*60);
  }

  for (ct_AN=1; ct_AN<=atomnum; ct_AN++){
    FREAD(Gxyz[ct_AN],sizeof(double),4,fp);
  }

  /****************************************************
    allocation of arrays:

    Kohn-Sham Hamiltonian

    dooble Hks[SpinP_switch+1]
    [atomnum+1]
    [FNAN[ct_AN]+1]
    [Total_NumOrbs[ct_AN]]
    [Total_NumOrbs[h_AN]];

    Overlap matrix

    dooble OLP[atomnum+1]
    [FNAN[ct_AN]+1]
    [Total_NumOrbs[ct_AN]]
    [Total_NumOrbs[h_AN]]; 

    Overlap matrix with position operator x, y, z

    double ******OLPpo;
    [3]
    [1]
    [atomnum+1]
    [FNAN[ct_AN]+1]
    [Total_NumOrbs[ct_AN]]
    [Total_NumOrbs[h_AN]]; 

    Overlap matrix with momentum operator px, py, pz

    double *****OLPmo;
    [3]
    [atomnum+1]
    [FNAN[ct_AN]+1]
    [Total_NumOrbs[ct_AN]]
    [Total_NumOrbs[h_AN]]; 

    Density matrix

    dooble DM[SpinP_switch+1]
    [atomnum+1]
    [FNAN[ct_AN]+1]
    [Total_NumOrbs[ct_AN]]
    [Total_NumOrbs[h_AN]];
   ****************************************************/

  Hks = (double*****)malloc(sizeof(double****)*(SpinP_switch+1));
  for (spin=0; spin<=SpinP_switch; spin++){

    Hks[spin] = (double****)malloc(sizeof(double***)*(atomnum+1));
    for (ct_AN=0; ct_AN<=atomnum; ct_AN++){
      TNO1 = Total_NumOrbs[ct_AN];
      Hks[spin][ct_AN] = (double***)malloc(sizeof(double**)*(FNAN[ct_AN]+1));
      for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){
	Hks[spin][ct_AN][h_AN] = (double**)malloc(sizeof(double*)*TNO1);

	if (ct_AN==0){
	  TNO2 = 1;
	}
	else{
	  Gh_AN = natn[ct_AN][h_AN];
	  TNO2 = Total_NumOrbs[Gh_AN];
	}
	for (i=0; i<TNO1; i++){
	  Hks[spin][ct_AN][h_AN][i] = (double*)malloc(sizeof(double)*TNO2);
	}
      }
    }
  }

  //printf("Hks col alloc OK!\n");

  /* Added by N. Yamaguchi ***/
  if (SpinP_switch==3){
    /* ***/

    iHks = (double*****)malloc(sizeof(double****)*3);
    for (spin=0; spin<3; spin++){

      iHks[spin] = (double****)malloc(sizeof(double***)*(atomnum+1));
      for (ct_AN=0; ct_AN<=atomnum; ct_AN++){
	TNO1 = Total_NumOrbs[ct_AN];
	iHks[spin][ct_AN] = (double***)malloc(sizeof(double**)*(FNAN[ct_AN]+1));
	for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){
	  iHks[spin][ct_AN][h_AN] = (double**)malloc(sizeof(double*)*TNO1);

	  if (ct_AN==0){
	    TNO2 = 1;
	  }
	  else{
	    Gh_AN = natn[ct_AN][h_AN];
	    TNO2 = Total_NumOrbs[Gh_AN];
	  }
	  for (i=0; i<TNO1; i++){
	    iHks[spin][ct_AN][h_AN][i] = (double*)malloc(sizeof(double)*TNO2);
	    for (j=0; j<TNO2; j++) iHks[spin][ct_AN][h_AN][i][j] = 0.0;
	  }
	}
      }
    }

  //printf("Hks non-col alloc OK!\n");

    /* Added by N. Yamaguchi ***/
  }
  /* ***/

  OLP = (double****)malloc(sizeof(double***)*(atomnum+1));
  for (ct_AN=0; ct_AN<=atomnum; ct_AN++){
    TNO1 = Total_NumOrbs[ct_AN];
    OLP[ct_AN] = (double***)malloc(sizeof(double**)*(FNAN[ct_AN]+1));
    for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){
      OLP[ct_AN][h_AN] = (double**)malloc(sizeof(double*)*TNO1);

      if (ct_AN==0){
	TNO2 = 1;
      }
      else{
	Gh_AN = natn[ct_AN][h_AN];
	TNO2 = Total_NumOrbs[Gh_AN];
      }
      for (i=0; i<TNO1; i++){
	OLP[ct_AN][h_AN][i] = (double*)malloc(sizeof(double)*TNO2);
      }
    }
  }

  //printf("Overlap matrix read OK!\n");

  /* Added by N. Yamaguchi ***/

  OLPpo=(double******)malloc(sizeof(double*****)*3);
  for (direction=0; direction<3; direction++){
    OLPpo[direction]=(double*****)malloc(sizeof(double****)*order_max);
    for (order=0; order<order_max; order++){
      OLPpo[direction][order]=(double****)malloc(sizeof(double***)*(atomnum+1));
      for (ct_AN=0; ct_AN<=atomnum; ct_AN++){
	TNO1 = Total_NumOrbs[ct_AN];
	OLPpo[direction][order][ct_AN] = (double***)malloc(sizeof(double**)*(FNAN[ct_AN]+1));
	for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){
	  OLPpo[direction][order][ct_AN][h_AN] = (double**)malloc(sizeof(double*)*TNO1);

	  if (ct_AN==0){
	    TNO2 = 1;
	  }
	  else{
	    Gh_AN = natn[ct_AN][h_AN];
	    TNO2 = Total_NumOrbs[Gh_AN];
	  }
	  for (i=0; i<TNO1; i++){
	    OLPpo[direction][order][ct_AN][h_AN][i] = (double*)malloc(sizeof(double)*TNO2);
	  }
	}
      }
    }
  }

  OLPmo=(double*****)malloc(sizeof(double****)*3);
  for (direction=0; direction<3; direction++){
    OLPmo[direction]=(double****)malloc(sizeof(double***)*(atomnum+1));
    for (ct_AN=0; ct_AN<=atomnum; ct_AN++){
      TNO1 = Total_NumOrbs[ct_AN];
      OLPmo[direction][ct_AN] = (double***)malloc(sizeof(double**)*(FNAN[ct_AN]+1));
      for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){
	OLPmo[direction][ct_AN][h_AN] = (double**)malloc(sizeof(double*)*TNO1);

	if (ct_AN==0){
	  TNO2 = 1;
	}
	else{
	  Gh_AN = natn[ct_AN][h_AN];
	  TNO2 = Total_NumOrbs[Gh_AN];
	}
	for (i=0; i<TNO1; i++){
	  OLPmo[direction][ct_AN][h_AN][i] = (double*)malloc(sizeof(double)*TNO2);
	}
      }
    }
  }

  DM = (double*****)malloc(sizeof(double****)*(SpinP_switch+1));
  for (spin=0; spin<=SpinP_switch; spin++){

    DM[spin] = (double****)malloc(sizeof(double***)*(atomnum+1));
    for (ct_AN=0; ct_AN<=atomnum; ct_AN++){
      TNO1 = Total_NumOrbs[ct_AN];
      DM[spin][ct_AN] = (double***)malloc(sizeof(double**)*(FNAN[ct_AN]+1));
      for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){
	DM[spin][ct_AN][h_AN] = (double**)malloc(sizeof(double*)*TNO1);

	if (ct_AN==0){ 
	  TNO2 = 1;
	}
	else{ 
	  Gh_AN = natn[ct_AN][h_AN];
	  TNO2 = Total_NumOrbs[Gh_AN];
	}
	for (i=0; i<TNO1; i++){
	  DM[spin][ct_AN][h_AN][i] = (double*)malloc(sizeof(double)*TNO2);
	}
      }
    }
  }

  iDM = (double*****)malloc(sizeof(double****)*2);
  for (spin=0; spin<2; spin++){

    iDM[spin] = (double****)malloc(sizeof(double***)*(atomnum+1));
    for (ct_AN=0; ct_AN<=atomnum; ct_AN++){
      TNO1 = Total_NumOrbs[ct_AN];
      iDM[spin][ct_AN] = (double***)malloc(sizeof(double**)*(FNAN[ct_AN]+1));
      for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){
	iDM[spin][ct_AN][h_AN] = (double**)malloc(sizeof(double*)*TNO1);

	if (ct_AN==0){ 
	  TNO2 = 1;
	}
	else{ 
	  Gh_AN = natn[ct_AN][h_AN];
	  TNO2 = Total_NumOrbs[Gh_AN];
	}
	for (i=0; i<TNO1; i++){
	  iDM[spin][ct_AN][h_AN][i] = (double*)malloc(sizeof(double)*TNO2);
	}
      }
    }
  }

  /****************************************************
    Hamiltonian matrix
   ****************************************************/

  for (spin=0; spin<=SpinP_switch; spin++){
    for (ct_AN=1; ct_AN<=atomnum; ct_AN++){
      TNO1 = Total_NumOrbs[ct_AN];
      for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){
	Gh_AN = natn[ct_AN][h_AN];
	TNO2 = Total_NumOrbs[Gh_AN];
	for (i=0; i<TNO1; i++){
	  FREAD(Hks[spin][ct_AN][h_AN][i],sizeof(double),TNO2,fp);
	}
      }
    }
  }

  /*********************************************************
   iHks:
   imaginary Kohn-Sham matrix elements of basis orbitals
   for alpha-alpha, beta-beta, and alpha-beta spin matrices
   of which contributions come from spin-orbit coupling 
   and Hubbard U effective potential.
  **********************************************************/

  if (SpinP_switch==3){
    for (spin=0; spin<3; spin++){
      for (ct_AN=1; ct_AN<=atomnum; ct_AN++){
	TNO1 = Total_NumOrbs[ct_AN];
	for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){
	  Gh_AN = natn[ct_AN][h_AN];
	  TNO2 = Total_NumOrbs[Gh_AN];
	  for (i=0; i<TNO1; i++){
	    FREAD(iHks[spin][ct_AN][h_AN][i],sizeof(double),TNO2,fp);
	  }
	}
      }
    }
  }

  /****************************************************
      Overlap matrix
   ****************************************************/

  for (ct_AN=1; ct_AN<=atomnum; ct_AN++){
    TNO1 = Total_NumOrbs[ct_AN];
    for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){
      Gh_AN = natn[ct_AN][h_AN];
      TNO2 = Total_NumOrbs[Gh_AN];
      for (i=0; i<TNO1; i++){
	FREAD(OLP[ct_AN][h_AN][i],sizeof(double),TNO2,fp);
      }
    }
  }

  /****************************************************
    Overlap matrix with position operator
    (added by N. Yamaguchi for HWC)
   ****************************************************/

  /* Added by N. Yamaguchi ***/

  for (direction=0; direction<3; direction++){
    for (order=0; order<order_max; order++){
      for (ct_AN=1; ct_AN<=atomnum; ct_AN++){
	TNO1 = Total_NumOrbs[ct_AN];
	for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){
	  Gh_AN = natn[ct_AN][h_AN];
	  TNO2 = Total_NumOrbs[Gh_AN];
	  for (i=0; i<TNO1; i++){
	    FREAD(OLPpo[direction][order][ct_AN][h_AN][i],sizeof(double),TNO2,fp);
	  }
	}
      }
    }
  }

  /****************************************************
    Overlap matrix with momentum operator
   ****************************************************/

  for (direction=0; direction<3; direction++){
    for (ct_AN=1; ct_AN<=atomnum; ct_AN++){
      TNO1 = Total_NumOrbs[ct_AN];
      for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){
	Gh_AN = natn[ct_AN][h_AN];
	TNO2 = Total_NumOrbs[Gh_AN];
	for (i=0; i<TNO1; i++){
	  FREAD(OLPmo[direction][ct_AN][h_AN][i],sizeof(double),TNO2,fp);
	}
      }
    }
  }

  /****************************************************
             Density matrix: DM and iDM
   ****************************************************/

  for (spin=0; spin<=SpinP_switch; spin++){
    for (ct_AN=1; ct_AN<=atomnum; ct_AN++){
      TNO1 = Total_NumOrbs[ct_AN];
      for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){
	Gh_AN = natn[ct_AN][h_AN];
	TNO2 = Total_NumOrbs[Gh_AN];
	for (i=0; i<TNO1; i++){
	  FREAD(DM[spin][ct_AN][h_AN][i],sizeof(double),TNO2,fp);
	}
      }
    }
  }

  for (spin=0; spin<2; spin++){
    for (ct_AN=1; ct_AN<=atomnum; ct_AN++){
      TNO1 = Total_NumOrbs[ct_AN];
      for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){
	Gh_AN = natn[ct_AN][h_AN];
	TNO2 = Total_NumOrbs[Gh_AN];
	for (i=0; i<TNO1; i++){
	  FREAD(iDM[spin][ct_AN][h_AN][i],sizeof(double),TNO2,fp);
	}
      }
    }
  }

  /****************************************************
    Solver
   ****************************************************/

  FREAD(i_vec,sizeof(int),1,fp);
  Solver = i_vec[0];

  /****************************************************
    ChemP
    Temp
   ****************************************************/

  FREAD(d_vec,sizeof(double),10,fp);
  ChemP  = d_vec[0];
  E_Temp = d_vec[1];
  dipole_moment_core[1] = d_vec[2]; 
  dipole_moment_core[2] = d_vec[3]; 
  dipole_moment_core[3] = d_vec[4]; 
  dipole_moment_background[1] = d_vec[5]; 
  dipole_moment_background[2] = d_vec[6]; 
  dipole_moment_background[3] = d_vec[7]; 
  Valence_Electrons = d_vec[8]; 
  Total_SpinS = d_vec[9];

  /****************************************************
    input file
   ****************************************************/

  FREAD(i_vec, sizeof(int), 1, fp);
  num_lines = i_vec[0];

  sprintf(makeinp,"temporal_12345.input");

  if ((fp_makeinp = fopen(makeinp,"w")) != NULL){

    setvbuf(fp_makeinp,buf,_IOFBF,fp_bsize);  /* setvbuf */
    for (i=1; i<=num_lines; i++){
      FREAD(strg, sizeof(char), MAX_LINE_SIZE, fp);
      fprintf(fp_makeinp,"%s",strg);
    }
    fclose(fp_makeinp);
  }
}


void free_scfout()
{
  int Rn,ct_AN,i,h_AN,spin,TNO1,order,direction;

  for (Rn=0; Rn<=TCpyCell; Rn++){
    free(atv[Rn]);
  }
  free(atv);

  for (Rn=0; Rn<=TCpyCell; Rn++){
    free(atv_ijk[Rn]);
  }
  free(atv_ijk);

  for (ct_AN=0; ct_AN<=atomnum; ct_AN++){
    free(natn[ct_AN]);
  }
  free(natn);

  for (ct_AN=0; ct_AN<=atomnum; ct_AN++){
    free(ncn[ct_AN]);
  }
  free(ncn);

  for (i=0; i<(atomnum+1); i++){
    free(Gxyz[i]);
  }
  free(Gxyz);

  for (spin=0; spin<=SpinP_switch; spin++){

    for (ct_AN=0; ct_AN<=atomnum; ct_AN++){
      TNO1 = Total_NumOrbs[ct_AN];
      for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){
	for (i=0; i<TNO1; i++){
	  free(Hks[spin][ct_AN][h_AN][i]);
	}
        free(Hks[spin][ct_AN][h_AN]);
      }
      free(Hks[spin][ct_AN]);
    }
    free(Hks[spin]);
  }
  free(Hks);

  if (SpinP_switch==3){

    for (spin=0; spin<3; spin++){

      for (ct_AN=0; ct_AN<=atomnum; ct_AN++){
	TNO1 = Total_NumOrbs[ct_AN];
	for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){
	  for (i=0; i<TNO1; i++){
	    free(iHks[spin][ct_AN][h_AN][i]);
	  }
          free(iHks[spin][ct_AN][h_AN]);
	}
        free(iHks[spin][ct_AN]);
      }
      free(iHks[spin]);
    }
    free(iHks);
  }

  for (ct_AN=0; ct_AN<=atomnum; ct_AN++){
    TNO1 = Total_NumOrbs[ct_AN];
    for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){
      for (i=0; i<TNO1; i++){
	free(OLP[ct_AN][h_AN][i]);
      }
      free(OLP[ct_AN][h_AN]);
    }
    free(OLP[ct_AN]);
  }
  free(OLP);

  for (direction=0; direction<3; direction++){
    for (order=0; order<order_max; order++){
      for (ct_AN=0; ct_AN<=atomnum; ct_AN++){
	TNO1 = Total_NumOrbs[ct_AN];
	for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){
	  for (i=0; i<TNO1; i++){
	    free(OLPpo[direction][order][ct_AN][h_AN][i]);
	  }
          free(OLPpo[direction][order][ct_AN][h_AN]);
	}
        free(OLPpo[direction][order][ct_AN]);
      }
      free(OLPpo[direction][order]);
    }
    free(OLPpo[direction]);
  }
  free(OLPpo);

  for (direction=0; direction<3; direction++){
    for (ct_AN=0; ct_AN<=atomnum; ct_AN++){
      TNO1 = Total_NumOrbs[ct_AN];
      for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){
	for (i=0; i<TNO1; i++){
	  free(OLPmo[direction][ct_AN][h_AN][i]);
	}
        free(OLPmo[direction][ct_AN][h_AN]);
      }
      free(OLPmo[direction][ct_AN]);
    }
    free(OLPmo[direction]);
  }
  free(OLPmo);

  for (spin=0; spin<=SpinP_switch; spin++){
    for (ct_AN=0; ct_AN<=atomnum; ct_AN++){
      TNO1 = Total_NumOrbs[ct_AN];
      for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){
	for (i=0; i<TNO1; i++){
	  free(DM[spin][ct_AN][h_AN][i]);
	}
        free(DM[spin][ct_AN][h_AN]);
      }
      free(DM[spin][ct_AN]);
    }
    free(DM[spin]);
  }
  free(DM);

  for (spin=0; spin<2; spin++){
    for (ct_AN=0; ct_AN<=atomnum; ct_AN++){
      TNO1 = Total_NumOrbs[ct_AN];
      for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){
	for (i=0; i<TNO1; i++){
	  free(iDM[spin][ct_AN][h_AN][i]);
	}
        free(iDM[spin][ct_AN][h_AN]);
      }
      free(iDM[spin][ct_AN]);
    }
    free(iDM[spin]);
  }
  free(iDM);

  free(Total_NumOrbs);
  free(FNAN);
}

void prepare_HSR(){
  static int i,j,l,n, II, JJ;
  static int ct_AN,h_AN,Gh_AN;
  static int spin,Rn, iR;
  int end_switch;    /* switch in the evalusion loop of J */


  MP = (int*)malloc(sizeof(int)*(atomnum+1));
  T_NumOrbs = 0;
  for (i=0; i<atomnum; i++){
    ct_AN = i+1;
    MP[i+1] = T_NumOrbs+1;
    T_NumOrbs = T_NumOrbs + Total_NumOrbs[ct_AN];
  }


  // allocate HR
  //[iR, spin,  iorb, jorb]
  HR = (double****)malloc(sizeof(double***)*(TCpyCell+1));
  for (iR=0; iR<TCpyCell+1; iR++){
    HR[iR]= (double***)malloc(sizeof(double**)*(SpinP_switch+1));
    for (spin=0; spin<=SpinP_switch; spin++){
      HR[iR][spin] = (double**)malloc(sizeof(double*)*(T_NumOrbs));
      for (i=0; i<T_NumOrbs; i++){
        HR[iR][spin][i] = (double*)malloc(sizeof(double)*(T_NumOrbs));
        for (j=0; j<T_NumOrbs; j++){
          HR[iR][spin][i][j]=0.0;
        }
      }
    }
  }

if(SpinP_switch==3) {
  HR_imag = (double****)malloc(sizeof(double***)*(TCpyCell+1));
  for (iR=0; iR<TCpyCell+1; iR++){
    HR_imag[iR]= (double***)malloc(sizeof(double**)*3);
    for (spin=0; spin<=2; spin++){
      HR_imag[iR][spin] = (double**)malloc(sizeof(double*)*(T_NumOrbs));
      for (i=0; i<T_NumOrbs; i++){
        HR_imag[iR][spin][i] = (double*)malloc(sizeof(double)*(T_NumOrbs));
        for (j=0; j<T_NumOrbs; j++){
          HR_imag[iR][spin][i][j]=0.0;
        }
      }
    }
  }
}


  // allocate SR
  //[iR, spin,  iorb, jorb]
  SR = (double***)malloc(sizeof(double***)*(TCpyCell+1));
  for (iR=0; iR<TCpyCell+1; iR++){
      SR[iR] = (double**)malloc(sizeof(double*)*(T_NumOrbs+1));
      for (i=0; i<=T_NumOrbs; i++){
        SR[iR][i] = (double*)malloc(sizeof(double)*(T_NumOrbs+1));
        for (j=0; j<=T_NumOrbs; j++){
          SR[iR][i][j]=0;

      }
    }
  }



// fill HR and SR
for (spin=0; spin<=SpinP_switch; spin++){
  for (II=0; II<atomnum; II++){
    for (JJ=0; JJ<atomnum; JJ++){
      ct_AN = II+1 ; //index of atom I
      // loop over NN list
      for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){ 
        // global index of NN atom J
        Gh_AN = natn[ct_AN][h_AN]; // why not h_AN+1?
        // or if JJ+1 in NN list
        if (Gh_AN == JJ+1){
          iR = ncn[ct_AN][h_AN];
          //if(iR==0 || iR==342){
          //   printf("iR: %d\n", iR);
          //}
          //printf("Norb for atom %d: %d\n", ct_AN, Total_NumOrbs[ct_AN]);
          for (i=0; i<Total_NumOrbs[ct_AN]; i++){
            //printf("iorb: %d\n", i+MP[II+1]);
            for (j=0; j<Total_NumOrbs[Gh_AN]; j++){
              HR[iR][spin][i+MP[II+1]-1][j+MP[JJ+1]-1]
                = Hks[spin][ct_AN][h_AN][i][j];
              SR[iR][i+MP[II+1]-1][j+MP[JJ+1]-1]
                = OLP[ct_AN][h_AN][i][j];
            }
          }
        }
      }
    }
  }
 }



// fill HR_imag
if (SpinP_switch==3){
for (spin=0; spin<3; spin++){
  for (II=0; II<atomnum; II++){
    for (JJ=0; JJ<atomnum; JJ++){
      ct_AN = II+1 ; //index of atom I
      // loop over NN list
      for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){ 
        // global index of NN atom J
        Gh_AN = natn[ct_AN][h_AN]; // why not h_AN+1?
        // or if JJ+1 in NN list
        if (Gh_AN == JJ+1){
          iR = ncn[ct_AN][h_AN];
          for (i=0; i<Total_NumOrbs[ct_AN]; i++){
            for (j=0; j<Total_NumOrbs[Gh_AN]; j++){
              HR_imag[iR][spin][i+MP[II+1]-1][j+MP[JJ+1]-1]
                = iHks[spin][ct_AN][h_AN][i][j];
            }
          }
        }
      }
    }
  }
 }
 }


  //printf("Hks nc read OK!\n");

}




void free_HSR(){
  int spin, i, iR;

  for (iR=0; iR<TCpyCell+1; iR++){
    for (spin=0; spin<=SpinP_switch; spin++){
      for (i=0; i<T_NumOrbs; i++){
        free(HR[iR][spin][i]);
      }
      free(HR[iR][spin]);
    }
    free(HR[iR]);
  }
  free(HR);



  if(SpinP_switch==3) {
  for (iR=0; iR<TCpyCell+1; iR++){
    for (spin=0; spin<=2; spin++){
      for (i=0; i<T_NumOrbs; i++){
        free(HR_imag[iR][spin][i]);
      }
      free(HR_imag[iR][spin]);
    }
    free(HR_imag[iR]);
  }
  free(HR_imag);
  }



  for (iR=0; iR<TCpyCell+1; iR++){
    for (i=0; i<T_NumOrbs; i++){
      free(SR[iR][i]);
    }
    free(SR[iR]);
  }
  free(SR);

  free(MP);
}


""")

ffi.cdef("""
        extern int atomnum;
        extern int Catomnum;
        extern int Ratomnum;
        extern int SpinP_switch;
        extern int TCpyCell;
        extern int Solver;
        extern double ChemP;
        extern int Valence_Electrons;
        extern double Total_SpinS;
        extern double E_Temp;
        extern int *Total_NumOrbs;
        extern int *FNAN;
        extern int **natn;
        extern int **ncn;
        extern double **atv;
        extern int **atv_ijk;
        extern double tv[4][4];
        extern double rtv[4][4];
        extern double **Gxyz;
        extern double *****Hks;
        extern double *****iHks;
        extern double ****OLP;
        extern double *****DM;
        extern double dipole_moment_core[4];
        extern int version;
        extern int order_max;
        extern double *cc_vec;
        extern double ******OLPpo;
        extern double *****OLPmo;
        extern double dipole_moment_background[4];
        extern int T_NumOrbs; /* total number of orbs*/
        extern int *MP ;         /* Full system version of MP */
        extern double ****HR;
        extern double ****HR_imag;
        extern double ***SR;

        void read_scfout(char *argv[]);
        void free_scfout();
        void prepare_HSR();
        void free_HSR();

        """)
if __name__ == '__main__':
    ffi.compile(verbose=True)



#!/usr/bin/env python3
import argparse
from TB2J.versioninfo import print_license
from TB2J_OpenMX.gen_exchange import gen_exchange

def run_openmx2J():
    print_license()
    print("\n")
    parser = argparse.ArgumentParser(
        description=
        "openmx2J: Using magnetic force theorem to calculate exchange parameter J from openmx Hamiltonian"
    )
    parser.add_argument(
        '--prefix', help="prefix of the openmx files", default='openmx', type=str)
    parser.add_argument(
        '--elements',
        help="elements to be considered in Heisenberg model",
        default=None,
        type=str,
        nargs='*')
    parser.add_argument(
        '--rcut', help='range of R. The default is all the commesurate R to the kmesh', default=None, type=float)
    parser.add_argument(
        '--efermi', help='Fermi energy in eV', default=None, type=float)
    parser.add_argument(
        '--kmesh',
        help='kmesh in the format of kx ky kz. Monkhorst pack. If all the numbers are odd, it is Gamma cenetered. (strongly recommended)',
        type=int,
        nargs='*',
        default=[5, 5, 5])
    parser.add_argument(
        '--emin',
        help='energy minimum below efermi, default -14 eV',
        type=float,
        default=-14.0)
    parser.add_argument(
        '--emax',
        help='energy maximum above efermi, default 0.0 eV',
        type=float,
        default=0.05)
    parser.add_argument(
        '--use_cache',
        help="whether to use disk file for temporary storing wavefunctions and hamiltonian to reduce memory usage. Default: False",
        action='store_true',
        default=False)
    parser.add_argument(
        '--nz', help='number of integration steps, default: 50', default=50, type=int)

    parser.add_argument(
        '--exclude_orbs',
        help=
        "the indices of wannier functions to be excluded from magnetic site. counting start from 0",
        default=[],
        type=int,
        nargs='+')

    parser.add_argument(
        "--description",
        help=
        "add description of the calculatiion to the xml file. Essential information, like the xc functional, U values, magnetic state should be given.",
        type=str,
        default="Calculated with TB2J.\n")

    parser.add_argument(
        "--fname",
        default='exchange.xml',
        type=str,
        help='exchange xml file name. default: exchange.xml')

    args = parser.parse_args()

    if args.elements is None:
        print("Please input the magnetic elements, e.g. --elements Fe Ni")
        exit()
    gen_exchange(
        path='./',
        prefix=args.prefix,
        kmesh=args.kmesh,
        magnetic_elements=args.elements,
        Rcut=args.rcut,
        emin=args.emin,
        emax=args.emax,
        nz=args.nz,
        description=args.description,
        use_cache=args.use_cache,
        exclude_orbs=args.exclude_orbs)

if __name__ == "__main__":
    run_openmx2J()



import os
from cffi import FFI
import numpy as np
import scipy.linalg as sl
from ase.units import Ha, Bohr, Ry
from ase.io import read
from ase import Atoms
from TB2J.utils import kmesh_to_R, symbol_number
import matplotlib.pyplot as plt
from TB2J_OpenMX.cmod._scfout_parser import ffi, lib
import copy

## Create the dictionary mapping ctypes to np dtypes.
ctype2dtype = {'int': 'i4', 'double': 'f8'}

## Integer types
for prefix in ('int', 'uint'):
    for log_bytes in range(4):
        ctype = '%s%d_t' % (prefix, 8 * (2**log_bytes))
        dtype = '%s%d' % (prefix[0], 2**log_bytes)
        ctype2dtype[ctype] = np.dtype(dtype)

## Floating point types
ctype2dtype['float'] = np.dtype('f4')
ctype2dtype['double'] = np.dtype('f8')


def asarray(ffi, ptr, length):
    ## Get the canonical C type of the elements of ptr as a string.
    T = ffi.getctype(ffi.typeof(ptr).item)

    if T not in ctype2dtype:
        raise RuntimeError("Cannot create an array for element type: %s" % T)

    return np.frombuffer(ffi.buffer(ptr, length * ffi.sizeof(T)),
                         ctype2dtype[T])


class OpenmxWrapper():
    def __init__(self, path, prefix='openmx'):
        self.is_siesta = False
        self.is_orthogonal = False
        xyz_fname = os.path.join(path, prefix + '.xyz')
        fname = os.path.join(path, prefix + '.scfout')
        self.fname = fname
        self.R2kfactor = 2.0j * np.pi
        self.parse_scfoutput()
        self.Rdict = dict()
        for i, R in enumerate(self.R):
            self.Rdict[tuple(R)] = i
        atoms = read(xyz_fname)
        self.atoms = Atoms(atoms.get_chemical_symbols(),
                           cell=self.cell,
                           positions=self.positions)
        self.norbs_to_basis(self.atoms, self.norbs)
        self.nspin = 2
        self.nbasis = self.nspin * self.norb

    def solve(self, k):
        phase = np.exp(self.R2kfactor * (self.R @ k))
        Hk = np.einsum('rij, r->ij', self.H, phase)
        Sk = np.einsum('rij, r->ij', self.S, phase)
        return sl.eigh(Hk, Sk)

    def solve_all(self, kpts):
        nk = len(kpts)
        evals = np.zeros((nk, self.norb))
        evecs = np.zeros((nk, self.norb, self.norb), dtype=complex)
        for ik, k in enumerate(kpts):
            evals[ik], evecs[ik] = self.solve(k)
        return evals, evecs

    def HSE_k(self, k, convention=2):
        phase = np.exp(self.R2kfactor * (self.R @ k))
        Hk = np.einsum('rij, r->ij', self.H, phase)
        Sk = np.einsum('rij, r->ij', self.S, phase)
        evalue, evec = sl.eigh(Hk, Sk)
        return Hk, Sk, evalue, evec


    def HS_and_eigen(self, kpts):
        nk = len(kpts)
        evals = np.zeros((nk, self.nbasis))
        evecs = np.zeros((nk, self.nbasis, self.nbasis), dtype=complex)
        Hk = np.zeros((nk, self.nbasis, self.nbasis), dtype=complex)
        Sk = np.zeros((nk, self.nbasis, self.nbasis), dtype=complex)
        for ik, k in enumerate(kpts):
            phase = np.exp(self.R2kfactor * (self.R @ k))
            Hk[ik] = np.einsum('rij, r->ij', self.H, phase)
            Sk[ik] = np.einsum('rij, r->ij', self.S, phase)
            evals[ik], evecs[ik] = sl.eigh(Hk[ik], Sk[ik])
        return Hk, Sk, evals, evecs

    def get_hamR(self, R):
        return self.H[self.Rdict[tuple(R)]]

    def norbs_to_basis(self, atoms, norbs):
        self.basis = []
        symbols = atoms.get_chemical_symbols()
        sn = list(symbol_number(symbols).keys())
        for i, n in enumerate(norbs):
            for x in range(n):
                self.basis .append((sn[i], f'orb{x+1}', 'up'))
                self.basis .append((sn[i], f'orb{x+1}', 'down'))
        return self.basis

    def parse_scfoutput(self):
        argv0 = ffi.new("char[]", b"")
        argv = ffi.new("char[]", bytes(self.fname, encoding='ascii'))
        lib.read_scfout([argv0, argv])
        lib.prepare_HSR()
        self.ncell = lib.TCpyCell + 1
        self.natom = lib.atomnum
        self.norbs = np.copy(
            asarray(ffi, lib.Total_NumOrbs, self.natom + 1)[1:])

        if lib.SpinP_switch == 3:
            self.non_collinear = True
        elif lib.SpinP_switch == 1:
            self.non_collinear = False
        else:
            raise ValueError(
                " The value of SpinP_switch is %s. Can only get J from collinear and non-collinear mode."%lib.SpinP_switch)

        fnan = asarray(ffi, lib.FNAN, self.natom + 1)

        natn = []
        for iatom in range(self.natom):
            natn.append(asarray(ffi, lib.natn[iatom + 1], fnan[iatom + 1] + 1))

        ncn = []
        for iatom in range(self.natom):
            ncn.append(asarray(ffi, lib.ncn[iatom + 1], fnan[iatom + 1] + 1))
        # atv
        #  x,y,and z-components of translation vector of
        #periodically copied cells
        #size: atv[TCpyCell+1][4];
        atv = []
        for icell in range(self.ncell):
            atv.append(asarray(ffi, lib.atv[icell], 4))
        atv = np.copy(np.array(atv))

        atv_ijk = []
        for icell in range(self.ncell):
            atv_ijk.append(asarray(ffi, lib.atv_ijk[icell], 4))
        atv_ijk = np.array(atv_ijk)
        self.R = atv_ijk[:, 1:]
        tv = []
        for i in range(4):
            tv.append(asarray(ffi, lib.tv[i], 4))
        tv = np.array(tv)
        self.cell = np.copy(tv[1:, 1:]) * Bohr

        rtv = []
        for i in range(4):
            rtv.append(asarray(ffi, lib.rtv[i], 4))
        rtv = np.array(rtv)
        self.rcell = np.copy(rtv[1:, 1:])

        Gxyz = []
        for iatom in range(self.natom):
            Gxyz.append(asarray(ffi, lib.Gxyz[iatom + 1], 60))
        self.positions = np.copy(np.array(Gxyz)[:, 1:4]) * Bohr

        self.MP = np.copy(asarray(ffi, lib.MP, self.natom + 1)[1:])

        self.norb = lib.T_NumOrbs
        norb = self.norb

        if self.non_collinear:
            HR = np.zeros([self.ncell, 4, lib.T_NumOrbs, lib.T_NumOrbs])
            for iR in range(0, self.ncell):
                for ispin in range(lib.SpinP_switch + 1):
                    for iorb in range(lib.T_NumOrbs):
                        HR[iR, ispin,
                           iorb, :] = asarray(ffi, lib.HR[iR][ispin][iorb],
                                              norb)

            HR_imag = np.zeros([self.ncell, 3, lib.T_NumOrbs, lib.T_NumOrbs])
            for iR in range(0, self.ncell):
                for ispin in range(3):
                    for iorb in range(lib.T_NumOrbs):
                        HR_imag[iR, ispin, iorb, :] = asarray(
                            ffi, lib.HR_imag[iR][ispin][iorb], norb)

            self.H = np.zeros(
                [self.ncell, lib.T_NumOrbs * 2, lib.T_NumOrbs * 2],
                dtype=complex)

            # up up
            for iR in range(self.ncell):
                self.H[iR, ::2, ::2] = HR[iR, 0, :, :] + 1j * HR_imag[iR, 0, :, :]
                # up down
                self.H[iR, ::2,
                   1::2] = HR[iR, 2, :, :] + 1j * (HR[iR, 3, :, :] +
                                                   HR_imag[iR, 2, :, :])
                # down up
                self.H[iR,
                   1::2, ::2] = HR[iR, 2, :, :] - 1j * (HR[iR, 3, :, :] +
                                                          HR_imag[iR, 2, :, :])
                # down down
                self.H[iR, 1::2, 1::2] = HR[iR, 1, :, :] + 1j * HR_imag[iR, 1, :, :]
        else:  # collinear

            HR = np.zeros([self.ncell, 4, lib.T_NumOrbs, lib.T_NumOrbs])
            for iR in range(0, self.ncell):
                for ispin in range(lib.SpinP_switch + 1):
                    for iorb in range(lib.T_NumOrbs):
                        HR[iR, ispin,
                           iorb, :] = asarray(ffi, lib.HR[iR][ispin][iorb],
                                              norb)

            self.H = np.zeros(
                [self.ncell, lib.T_NumOrbs * 2, lib.T_NumOrbs * 2],
                dtype=complex)

            # up up
            for iR in range(self.ncell):
                self.H[iR, ::2, ::2] = HR[iR, 0, :, :]
                self.H[iR, 1::2, 1::2] = HR[iR, 1, :, :]
        self.efermi = lib.ChemP * Ha
        self.H *= Ha

        SR = np.zeros([self.ncell, lib.T_NumOrbs, lib.T_NumOrbs])
        for iR in range(0, self.ncell):
            for iorb in range(lib.T_NumOrbs):
                SR[iR, iorb, :] = asarray(ffi, lib.SR[iR][iorb], norb)
        self.S = np.kron( SR, np.eye(2))
        lib.free_HSR()
        lib.free_scfout()
        print("Loading from scfout file OK!")

def test():
    openmx = OpenmxWrapper(
        path='/home/hexu/projects/TB2J_example/OPENMX/SrMnO3_FM_SOC/')
    #hsr = openmx.parse_scfoutput()
    #from banddownfolder.plot import plot_band
    #plot_band(hsr)
    #plt.savefig('band.pdf')
    #plt.show()


#test()



#!/usr/bin/env python
from setuptools import setup, find_packages
import sys

__version__ = "0.4.1"

long_description = """Interface of TB2J to OpenMX. 
TB2J is a Python package aimed to compute automatically the magnetic interactions (superexchange  and Dzyaloshinskii-Moriya) between atoms of magnetic crystals from DFT Hamiltonian based on Wannier functions or Linear combination of atomic orbitals. It uses the Green's function method and take the local rigid spin rotation as a perturbation. The package can take the output from Wannier90, which is interfaced with many density functional theory codes or from codes based on localised orbitals. A minimal user input is needed, which allows for an easily integration into a high-throughput workflows.
 """

setup(
    name='TB2J_OpenMX',
    version=__version__,
    description=
    'TB2J_OpenMX: TB2J interface to OpenMX',
    long_description=long_description,
    author='Xu He',
    author_email='mailhexu@gmail.com',
    license='GPLv3',
    packages=find_packages(),
    cffi_modules=["TB2J_OpenMX/ffimod.py:ffi"],
    scripts=["scripts/openmx2J.py"],
    install_requires=['numpy', 'scipy', 'matplotlib', 'ase', 'progressbar', 'TB2J>=0.6.0', 'cffi'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 3',
        #'Operating System :: OS Independent',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Chemistry',
        'Topic :: Scientific/Engineering :: Physics',
    ],
    python_requires='>=3.6',
)



from TB2J_OpenMX.ffiparser import OpenmxWrapper
from TB2J.exchange import ExchangeNCL, ExchangeCL
import os


def gen_exchange(path,
                 prefix='openmx',
                 magnetic_elements=[],
                 kmesh=[5, 5, 5],
                 emin=-11.0,
                 emax=0.00,
                 nz=100,
                 exclude_orbs=[],
                 Rcut=None,
                 use_cache=False,
                 description=None):
    tbmodel=OpenmxWrapper(path, prefix)
    if tbmodel.non_collinear:
        Exchange=ExchangeNCL
    else:
        Exchange=ExchangeCL
    print("Starting to calculate exchange.")
    description=f"""Using OpenMX data: 
path: {os.path.abspath(path)}
prefix: {prefix}
"""
    exchange = Exchange(
            tbmodels=tbmodel,
            atoms=tbmodel.atoms,
            basis=tbmodel.basis,
            efermi=tbmodel.efermi,
            magnetic_elements=magnetic_elements,
            kmesh=kmesh,
            emin=emin,
            emax=emax,
            nz=nz,
            exclude_orbs=exclude_orbs,
            Rcut=Rcut,
            use_cache=use_cache,
            description=description)
    exchange.run()
    print("\n")
    print("All calculation finsihed. The results are in TB2J_results directory.")

if __name__=='__main__':
    gen_exchange(
        path='/home/hexu/projects/TB2J_example/OPENMX/SrMnO3_FM_SOC/', magnetic_elements=['Mn'], nz=50, Rcut=8)
